@@ -2,32 +2,27 @@ import numpy as np
 from pathlib import Path
 import xgboost as xgb
 import lightgbm as lgb
-from typing import Literal, Union, Tuple, Dict, Optional, Any
-import pandas as pd
+from typing import Literal, Union, Tuple, Dict, Optional
 from copy import deepcopy
 from .utilities import (
     threshold_binary_values, 
     threshold_binary_values_batch, 
-    deserialize_object,
-    yield_dataframes_from_dir)
-from .path_manager import sanitize_filename, make_fullpath, list_files_by_extension, list_csv_paths
+    deserialize_object)
+from .path_manager import sanitize_filename, make_fullpath, list_files_by_extension
 import torch
 from tqdm import trange
-import matplotlib.pyplot as plt
-import seaborn as sns
 from ._logger import _LOGGER
-from .keys import ModelSaveKeys
+from .keys import EnsembleKeys
 from ._script_info import _script_info
 from .SQL import DatabaseManager
 from contextlib import nullcontext
+from .optimization_tools import _save_result
 
 
 __all__ = [
     "ObjectiveFunction",
     "multiple_objective_functions_from_dir",
-    "parse_lower_upper_bounds",
-    "run_pso",
-    "plot_optimal_feature_distributions"
+    "run_pso"
 ]
 
 
@@ -53,9 +48,9 @@ class ObjectiveFunction():
         self.is_hybrid = False if binary_features <= 0 else True
         self.use_noise = add_noise
         self._artifact = deserialize_object(trained_model_path, verbose=False, raise_on_error=True)
-        self.model = self._get_from_artifact(ModelSaveKeys.MODEL)
-        self.feature_names: Optional[list[str]] = self._get_from_artifact(ModelSaveKeys.FEATURES) # type: ignore
-        self.target_name: Optional[str] = self._get_from_artifact(ModelSaveKeys.TARGET) # type: ignore
+        self.model = self._get_from_artifact(EnsembleKeys.MODEL)
+        self.feature_names: Optional[list[str]] = self._get_from_artifact(EnsembleKeys.FEATURES) # type: ignore
+        self.target_name: Optional[str] = self._get_from_artifact(EnsembleKeys.TARGET) # type: ignore
         self.task = task
         self.check_model() # check for classification models and None values
         
@@ -131,7 +126,7 @@ class ObjectiveFunction():
         if self._artifact is None:
             raise TypeError("Load model error")
         val = self._artifact.get(key)
-        if key == ModelSaveKeys.FEATURES:
+        if key == EnsembleKeys.FEATURES:
             result = val if isinstance(val, list) and val else None
         else:
             result = val if val else None
@@ -170,18 +165,6 @@ def multiple_objective_functions_from_dir(directory: Union[str,Path], add_noise:
     return objective_functions, objective_function_names
 
 
-def parse_lower_upper_bounds(source: dict[str,tuple[Any,Any]]):
-    """
-    Parse lower and upper boundaries, returning 2 lists:
-    
-    `lower_bounds`, `upper_bounds`
-    """
-    lower = [low[0] for low in source.values()]
-    upper = [up[1] for up in source.values()]
-    
-    return lower, upper
-
-
 def _set_boundaries(lower_boundaries: list[float], upper_boundaries: list[float]):
     assert len(lower_boundaries) == len(upper_boundaries), "Lower and upper boundaries must have the same length."
     assert len(lower_boundaries) >= 1, "At least one boundary pair is required."
@@ -198,45 +181,6 @@ def _set_feature_names(size: int, names: Union[list[str], None]):
         return names
 
 
-def _save_result(result_dict: dict,
-                 save_format: Literal['csv', 'sqlite', 'both'],
-                 csv_path: Path,
-                 db_manager: Optional[DatabaseManager] = None,
-                 db_table_name: Optional[str] = None):
-    """
-    Handles saving a single result to CSV, SQLite, or both.
-    """
-    # Save to CSV
-    if save_format in ['csv', 'both']:
-        _save_or_append_to_csv(result_dict, csv_path)
-
-    # Save to SQLite
-    if save_format in ['sqlite', 'both']:
-        if db_manager and db_table_name:
-            db_manager.insert_row(db_table_name, result_dict)
-        else:
-            _LOGGER.warning("SQLite saving requested but db_manager or table_name not provided.")
-
-
-def _save_or_append_to_csv(data_dict: dict, save_path: Path):
-    """
-    Saves or appends a dictionary of data as a single row to a CSV file.
-
-    If the file doesn't exist, it creates it and writes the header.
-    If the file exists, it appends the new data without the header.
-    """
-    df_row = pd.DataFrame([data_dict])
-    
-    file_exists = save_path.exists()
-    
-    df_row.to_csv(
-        save_path, 
-        mode='a',              # 'a' for append mode
-        index=False,           # Don't write the DataFrame index
-        header=not file_exists # Write header only if file does NOT exist
-    )
-
-
 def _run_single_pso(objective_function: ObjectiveFunction, pso_args: dict, feature_names: list[str], target_name: str, random_state: int, save_format: Literal['csv', 'sqlite', 'both'], csv_path: Path, db_manager: Optional[DatabaseManager], db_table_name: str):
     """Helper for a single PSO run that also handles saving."""
     pso_args.update({"seed": random_state})
@@ -282,14 +226,14 @@ def run_pso(lower_boundaries: list[float],
             upper_boundaries: list[float], 
             objective_function: ObjectiveFunction,
             save_results_dir: Union[str,Path],
-            save_format: Literal['csv', 'sqlite', 'both'] = 'csv',
+            save_format: Literal['csv', 'sqlite', 'both'],
             auto_binary_boundaries: bool=True,
             target_name: Union[str, None]=None, 
             feature_names: Union[list[str], None]=None,
             swarm_size: int=200, 
             max_iterations: int=3000,
             random_state: int=101,
-            post_hoc_analysis: Optional[int]=10) -> Optional[Tuple[Dict[str, float], Dict[str, float]]]:
+            post_hoc_analysis: Optional[int]=20) -> Optional[Tuple[Dict[str, float], Dict[str, float]]]:
     """
     Executes Particle Swarm Optimization (PSO) to optimize a given objective function and saves the results as a CSV file.
 
@@ -303,7 +247,7 @@ def run_pso(lower_boundaries: list[float],
         A callable object encapsulating a tree-based regression model.
     save_results_dir : str | Path
         Directory path to save the results CSV file.
-    save_format : {'csv', 'sqlite', 'both'}, default 'csv'
+    save_format : {'csv', 'sqlite', 'both'}
         The format for saving optimization results.
         - 'csv': Saves results to a CSV file.
         - 'sqlite': Saves results to an SQLite database file. ⚠️ If a database exists, new tables will be created using the target name.
@@ -578,83 +522,6 @@ def _pso(func: ObjectiveFunction,
         return best_position, best_score
 
 
-def plot_optimal_feature_distributions(results_dir: Union[str, Path], save_dir: Union[str, Path]):
-    """
-    Analyzes optimization results and plots the distribution of optimal values for each feature.
-
-    For features with more than two unique values, this function generates a color-coded 
-    Kernel Density Estimate (KDE) plot. For binary or constant features, it generates a bar plot
-    showing relative frequency.
-
-    Parameters
-    ----------
-    results_dir : str or Path
-        The path to the directory containing the optimization result CSV files.
-    save_dir : str or Path
-        The directory where the output plots will be saved.
-    """
-    # Check results_dir and create output path
-    results_path = make_fullpath(results_dir)
-    output_path = make_fullpath(save_dir, make=True)
-    
-    # Check that the directory contains csv files
-    list_csv_paths(results_path, verbose=False)
-
-    # --- Data Loading and Preparation ---
-    _LOGGER.info(f"📁 Starting analysis from results in: '{results_dir}'")
-    data_to_plot = []
-    for df, df_name in yield_dataframes_from_dir(results_path):
-        melted_df = df.iloc[:, :-1].melt(var_name='feature', value_name='value')
-        melted_df['target'] = df_name.replace("Optimization_", "")
-        data_to_plot.append(melted_df)
-    
-    long_df = pd.concat(data_to_plot, ignore_index=True)
-    features = long_df['feature'].unique()
-    _LOGGER.info(f"📂 Found data for {len(features)} features across {len(long_df['target'].unique())} targets. Generating plots...")
-
-    # --- Plotting Loop ---
-    for feature_name in features:
-        plt.figure(figsize=(12, 7))
-        feature_df = long_df[long_df['feature'] == feature_name]
-
-        # Check if the feature is binary or constant
-        if feature_df['value'].nunique() <= 2:
-            # PLOT 1: For discrete values, calculate percentages and use a true bar plot.
-            # This ensures the X-axis is clean (e.g., just 0 and 1).
-            norm_df = (feature_df.groupby('target')['value']
-                       .value_counts(normalize=True)
-                       .mul(100)
-                       .rename('percent')
-                       .reset_index())
-            
-            ax = sns.barplot(data=norm_df, x='value', y='percent', hue='target')
-            
-            plt.title(f"Optimal Value Distribution for '{feature_name}'", fontsize=16)
-            plt.ylabel("Frequency (%)", fontsize=12)
-            ax.set_ylim(0, 100) # Set Y-axis from 0 to 100
-
-        else:
-            # PLOT 2: KDE plot for continuous values.
-            ax = sns.kdeplot(data=feature_df, x='value', hue='target',
-                             fill=True, alpha=0.1, warn_singular=False)
-
-            plt.title(f"Optimal Value Distribution for '{feature_name}'", fontsize=16)
-            plt.ylabel("Density", fontsize=12) # Y-axis is "Density" for KDE plots
-
-        # --- Common settings for both plot types ---
-        plt.xlabel("Feature Value", fontsize=12)
-        plt.grid(axis='y', alpha=0.5, linestyle='--')
-        
-        legend = ax.get_legend()
-        if legend:
-            legend.set_title('Target')
-
-        sanitized_feature_name = sanitize_filename(feature_name)
-        plot_filename = output_path / f"Distribution_{sanitized_feature_name}.svg"
-        plt.savefig(plot_filename, bbox_inches='tight')
-        plt.close()
-
-    _LOGGER.info(f"✅ All plots saved successfully to: '{output_path}'")
 
 
 def info():
