@@ -2,7 +2,9 @@ from pathlib import Path
 from openpyxl import load_workbook, Workbook
 import pandas as pd
 from typing import List, Optional, Union
-from .utilities import _script_info, sanitize_filename, make_fullpath
+from .path_manager import sanitize_filename, make_fullpath
+from ._script_info import _script_info
+from ._logger import _LOGGER
 
 
 __all__ = [
@@ -35,7 +37,7 @@ def find_excel_files(
     input_path = make_fullpath(directory)
 
     if not input_path.is_dir():
-        raise NotADirectoryError(f"Directory not found: {input_path}")
+        raise NotADirectoryError(f"❌ Directory not found: {input_path}")
 
     excel_files = [
         f for f in input_path.iterdir()
@@ -45,7 +47,7 @@ def find_excel_files(
     ]
     
     if not excel_files:
-        raise FileNotFoundError(f"No valid Excel files found in directory: {input_path}")
+        raise FileNotFoundError(f"❌ No valid Excel files found in directory: {input_path}")
 
     return excel_files
 
@@ -95,10 +97,9 @@ def unmerge_and_split_excel(filepath: Union[str,Path]) -> None:
         output_path = base_dir / output_filename
         new_wb.save(output_path)
 
-        # print(f"Saved: {output_path}")
         total_output_files += 1
 
-    print(f"✅ Processed file: {file_path} into {total_output_files} output file(s).")
+    _LOGGER.info(f"✅ Processed file: {file_path} into {total_output_files} output file(s).")
     return None
 
 
@@ -152,10 +153,9 @@ def unmerge_and_split_from_directory(input_dir: Union[str,Path], output_dir: Uni
             output_path = global_output_path / output_filename
             new_wb.save(output_path)
 
-            # print(f"Saved: {output_path}")
             total_output_files += 1
 
-    print(f"✅ Processed {len(excel_files)} input Excel file(s) with a total of {total_output_files} output Excel file(s).")
+    _LOGGER.info(f"✅ Processed {len(excel_files)} input Excel file(s) with a total of {total_output_files} output Excel file(s).")
     return None
 
 
@@ -199,13 +199,13 @@ def validate_excel_schema(
                     invalid_files.append(file)
 
         except Exception as e:
-            print(f"Error processing '{file}': {e}")
+            _LOGGER.error(f"❌ Error processing '{file}': {e}")
             invalid_files.append(file)
     
     valid_excel_number = len(excel_paths) - len(invalid_files)
-    print(f"{valid_excel_number} out of {len(excel_paths)} excel files conform to the schema.")
+    _LOGGER.info(f"{valid_excel_number} out of {len(excel_paths)} excel files conform to the schema.")
     if invalid_files:
-        print(f"⚠️ {len(invalid_files)} excel files are invalid:")
+        _LOGGER.warning(f"⚠️ {len(invalid_files)} excel files are invalid:")
         for in_file in invalid_files:
             print(f"  - {in_file.name}")
 
@@ -252,7 +252,7 @@ def vertical_merge_transform_excel(
         if target_columns is not None:
             missing = [col for col in target_columns if col not in df.columns]
             if missing:
-                raise ValueError(f"Invalid columns in {file.name}: {missing}")
+                raise ValueError(f"❌ Invalid columns in {file.name}: {missing}")
             df = df[target_columns]
 
         dataframes.append(df)
@@ -262,11 +262,11 @@ def vertical_merge_transform_excel(
     if rename_columns is not None:
         expected_len = len(target_columns if target_columns is not None else merged_df.columns)
         if len(rename_columns) != expected_len:
-            raise ValueError("Length of 'rename_columns' must match the selected columns")
+            raise ValueError("❌ Length of 'rename_columns' must match the selected columns")
         merged_df.columns = rename_columns
 
     merged_df.to_csv(csv_path, index=False, encoding='utf-8')
-    print(f"✅ Merged {len(dataframes)} excel files into '{csv_filename}'.")
+    _LOGGER.info(f"✅ Merged {len(dataframes)} excel files into '{csv_filename}'.")
 
 
 def horizontal_merge_transform_excel(
@@ -325,6 +325,9 @@ def horizontal_merge_transform_excel(
     merged_df = pd.concat(padded_dataframes, axis=1)
 
     duplicate_columns = merged_df.columns[merged_df.columns.duplicated()].tolist()
+    
+    if duplicate_columns:
+        _LOGGER.warning(f"⚠️ Duplicate columns: {duplicate_columns}")
 
     if skip_duplicates:
         merged_df = merged_df.loc[:, ~merged_df.columns.duplicated()]
@@ -344,10 +347,8 @@ def horizontal_merge_transform_excel(
 
     merged_df.to_csv(csv_path, index=False, encoding='utf-8')
 
-    print(f"✅ Merged {len(excel_files)} Excel files into '{csv_filename}'.")
-    if duplicate_columns:
-        print(f"⚠️ Duplicate columns: {duplicate_columns}")
-
+    _LOGGER.info(f"✅ Merged {len(excel_files)} Excel files into '{csv_filename}'.")
+    
 
 def info():
     _script_info(__all__)
